@@ -35,7 +35,7 @@ PA_C_DECL_BEGIN
 /** Return the version of the header files. Keep in mind that this is
 a macro and not a function, so it is impossible to get the pointer of
 it. */
-#define pa_get_headers_version() ("0.9.14")
+#define pa_get_headers_version() ("0.9.15")
 
 /** Return the version of the library the current application is
  * linked to. */
@@ -43,12 +43,22 @@ const char* pa_get_library_version(void);
 
 /** The current API version. Version 6 relates to Polypaudio
  * 0.6. Prior versions (i.e. Polypaudio 0.5.1 and older) have
- * PA_API_VERSION undefined.  */
+ * PA_API_VERSION undefined. Please note that this is only ever
+ * increased on incompatible API changes!  */
 #define PA_API_VERSION 12
 
 /** The current protocol version. Version 8 relates to Polypaudio
  * 0.8/PulseAudio 0.9. */
-#define PA_PROTOCOL_VERSION 14
+#define PA_PROTOCOL_VERSION 15
+
+/** The major version of PA. \since 0.9.15 */
+#define PA_MAJOR 0
+
+/** The minor version of PA. \since 0.9.15 */
+#define PA_MINOR 9
+
+/** The micro version of PA. \since 0.9.15 */
+#define PA_MICRO 15
 
 PA_C_DECL_END
 
