@@ -8,7 +8,7 @@
 
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
-  by the Free Software Foundation; either version 2 of the License,
+  by the Free Software Foundation; either version 2.1 of the License,
   or (at your option) any later version.
 
   PulseAudio is distributed in the hope that it will be useful, but
@@ -27,8 +27,9 @@
 
 typedef struct pa_module pa_module;
 
+#include <pulse/proplist.h>
+
 #include <pulsecore/core.h>
-#include <pulsecore/modinfo.h>
 
 struct pa_module {
     pa_core *core;
@@ -39,16 +40,14 @@ struct pa_module {
 
     int (*init)(pa_module*m);
     void (*done)(pa_module*m);
+    int (*get_n_used)(pa_module *m);
 
     void *userdata;
 
-    int n_used;
-
-    pa_bool_t auto_unload:1;
     pa_bool_t load_once:1;
     pa_bool_t unload_requested:1;
 
-    time_t last_used_time;
+    pa_proplist *proplist;
 };
 
 pa_module* pa_module_load(pa_core *c, const char *name, const char*argument);
@@ -60,9 +59,8 @@ void pa_module_unload_request(pa_module *m, pa_bool_t force);
 void pa_module_unload_request_by_index(pa_core *c, uint32_t idx, pa_bool_t force);
 
 void pa_module_unload_all(pa_core *c);
-void pa_module_unload_unused(pa_core *c);
 
-void pa_module_set_used(pa_module*m, int used);
+int pa_module_get_n_used(pa_module*m);
 
 #define PA_MODULE_AUTHOR(s)                                     \
     const char *pa__get_author(void) { return s; }              \
@@ -84,6 +82,4 @@ void pa_module_set_used(pa_module*m, int used);
     pa_bool_t pa__load_once(void) { return b; }                 \
     struct __stupid_useless_struct_to_allow_trailing_semicolon
 
-pa_modinfo *pa_module_get_info(pa_module *m);
-
 #endif
