@@ -8,7 +8,7 @@
 
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
-  by the Free Software Foundation; either version 2 of the License,
+  by the Free Software Foundation; either version 2.1 of the License,
   or (at your option) any later version.
 
   PulseAudio is distributed in the hope that it will be useful, but
@@ -30,18 +30,19 @@
 typedef enum pa_namereg_type {
     PA_NAMEREG_SINK,
     PA_NAMEREG_SOURCE,
-    PA_NAMEREG_SAMPLE
+    PA_NAMEREG_SAMPLE,
+    PA_NAMEREG_CARD
 } pa_namereg_type_t;
 
-void pa_namereg_free(pa_core *c);
-
 const char *pa_namereg_register(pa_core *c, const char *name, pa_namereg_type_t type, void *data, pa_bool_t fail);
 void pa_namereg_unregister(pa_core *c, const char *name);
-void* pa_namereg_get(pa_core *c, const char *name, pa_namereg_type_t type, pa_bool_t autoload);
-int pa_namereg_set_default(pa_core*c, const char *name, pa_namereg_type_t type);
+void* pa_namereg_get(pa_core *c, const char *name, pa_namereg_type_t type);
 
-const char *pa_namereg_get_default_sink_name(pa_core *c);
-const char *pa_namereg_get_default_source_name(pa_core *c);
+pa_sink* pa_namereg_set_default_sink(pa_core*c, pa_sink *s);
+pa_source* pa_namereg_set_default_source(pa_core*c, pa_source *s);
+
+pa_sink *pa_namereg_get_default_sink(pa_core *c);
+pa_source *pa_namereg_get_default_source(pa_core *c);
 
 pa_bool_t pa_namereg_is_valid_name(const char *name);
 char* pa_namereg_make_valid_name(const char *name);
