@@ -94,10 +94,11 @@ enum {
     PA_COMMAND_LOAD_MODULE,
     PA_COMMAND_UNLOAD_MODULE,
 
-    PA_COMMAND_ADD_AUTOLOAD,
-    PA_COMMAND_REMOVE_AUTOLOAD,
-    PA_COMMAND_GET_AUTOLOAD_INFO,
-    PA_COMMAND_GET_AUTOLOAD_INFO_LIST,
+    /* Obsolete */
+    PA_COMMAND_ADD_AUTOLOAD___OBSOLETE,
+    PA_COMMAND_REMOVE_AUTOLOAD___OBSOLETE,
+    PA_COMMAND_GET_AUTOLOAD_INFO___OBSOLETE,
+    PA_COMMAND_GET_AUTOLOAD_INFO_LIST___OBSOLETE,
 
     PA_COMMAND_GET_RECORD_LATENCY,
     PA_COMMAND_CORK_RECORD_STREAM,
@@ -151,6 +152,19 @@ enum {
     /* Supported since protocol v14 (0.9.12) */
     PA_COMMAND_EXTENSION,
 
+    /* Supported since protocol v15 (0.9.15) */
+    PA_COMMAND_GET_CARD_INFO,
+    PA_COMMAND_GET_CARD_INFO_LIST,
+    PA_COMMAND_SET_CARD_PROFILE,
+
+    PA_COMMAND_CLIENT_EVENT,
+    PA_COMMAND_PLAYBACK_STREAM_EVENT,
+    PA_COMMAND_RECORD_STREAM_EVENT,
+
+    /* SERVER->CLIENT */
+    PA_COMMAND_PLAYBACK_BUFFER_ATTR_CHANGED,
+    PA_COMMAND_RECORD_BUFFER_ATTR_CHANGED,
+
     PA_COMMAND_MAX
 };
 
